<?php

namespace frontend\controllers;

use common\models\PostEvent;
use frontend\models\Country;
use frontend\models\Currency;
use frontend\models\Events;
use Yii;
use frontend\models\EventsSearch;
use frontend\models\Region;
use frontend\models\Season;
use yii\web\BadRequestHttpException;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\Response;
use yii\web\UploadedFile;

/**
 * EventsController implements the CRUD actions for Events model.
 */
class EventsController extends Controller
{
    /**
     * @inheritDoc
     */
    public function behaviors()
    {
        return array_merge(
            parent::behaviors(),
            [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ]
        );
    }

    /**
     * @inheritdoc
     * @throws BadRequestHttpException
     */
    public function beforeAction($action)
    {
        if (Yii::$app->user->isGuest) {
            return $this->redirect(['/login']);
        }

        return parent::beforeAction($action);
    }

    /**
     * Lists all Events models.
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new EventsSearch();
        $dataProvider = $searchModel->search($this->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Events model.
     * @param int $id ID
     * @return string
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Events model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return Response
     */
    public function actionCreate()
    {
        $model = new PostEvent();

        if ($this->request->isPost) {
            if ($model->load($this->request->post()) ) {

                $model->cover = UploadedFile::getInstance($model, 'cover');
                $imageName = md5($model->cover->baseName.date('d-m-Y-H-i-s').Yii::$app->user->identity->id?? null);
                $path = 'uploads/images/' . $imageName . '.' . $model->cover->extension;
                $uploaded = $model->cover->saveAs($path);

                if($uploaded) {

                    $model->cover = $path;

                    if($model->validate() && $model->save()) {

                        Yii::$app->session->setFlash('success', 'Event saved successfully');
                        return $this->redirect(['view', 'id' => $model->id]);
                    }

                }

                Yii::$app->session->setFlash('error', 'Sorry!, Failed to save event, please try again');
//                return $this->redirect('create');

//                Yii::$app->response->format = Response::FORMAT_JSON;

            }
        } else {
            $model->loadDefaultValues();
        }

        return $this->render('create', [
            'model' => $model,
            'country' => Country::find()->where(['status' => 10])->all(),
            'season' => Season::find()->where(['status' => 10])->all(),
            'region' => Region::find()->where(['status' => 10])->all(),
            'currency' => Currency::find()->where(['status' => 10])->all(),
        ]);
    }

    /**
     * Updates an existing Events model.
     * If the update is successful, the browser will be redirected to the 'view' page.
     * @param int $id ID
     * @return string|Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($this->request->isPost && $model->load($this->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
            'country' => Country::find()->where(['status' => 10])->all(),
            'season' => Season::find()->where(['status' => 10])->all(),
            'region' => Region::find()->where(['status' => 10])->all(),
            'currency' => Currency::find()->where(['status' => 10])->all(),
        ]);
    }

    /**
     * Deletes an existing Events model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param int $id ID
     * @return Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Events model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param int $id ID
     * @return Events the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Events::findOne(['id' => $id])) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
