<?php

namespace frontend\controllers;

use frontend\models\Country;
use frontend\models\Events;
use frontend\models\Gallery;
use frontend\models\Headliner;
use frontend\models\HeadlinerAttributes;
use frontend\models\LineUp;
use frontend\models\LineUpList;
use frontend\models\RegistrationForm;
use frontend\models\ResendVerificationEmailForm;
use frontend\models\Schedule;
use frontend\models\ScheduleDays;
use frontend\models\ScheduleDaysResource;
use frontend\models\Tickets;
use frontend\models\VerifyEmailForm;
use Yii;
use yii\base\InvalidArgumentException;
use yii\db\Query;
use yii\web\BadRequestHttpException;
use yii\web\Controller;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use frontend\models\PasswordResetRequestForm;
use frontend\models\ResetPasswordForm;
use frontend\models\SignupForm;
use frontend\models\ContactForm;
use yii\web\Response;

/**
 * Site controller
 */
class SiteController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'only' => ['logout', 'signup'],
                'rules' => [
                    [
                        'actions' => ['signup'],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::class,
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => \yii\web\ErrorAction::class,
            ],
            'captcha' => [
                'class' => \yii\captcha\CaptchaAction::class,
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */
    public function actionIndex()
    {

        $this->layout = 'main-site';

        $model = Events::findOne([ 'status' => 10 ]);
        $tickets = Tickets::find()->all();
        $headliner = Headliner::find()->where(['status' => 10])->orderBy('id DESC')->one();
        $awards = HeadlinerAttributes::find()->where(['status' => 10])->orderBy('order DESC')->all();
        $lineup = LineUp::find()->where(['status' => 10])->orderBy('id DESC')->one();
        $schedule = Schedule::find()->where(['status' => 10])->orderBy('id DESC')->one();
        $days = ScheduleDaysResource::find()->where(['status' => 10])->all();
        $lineupList = LineUpList::find()->where(['status' => 10])->all();
        $amazing = Gallery::find()->where('id % 2 = 0')->all();
        $amazingTwo = Gallery::find()->where('id % 2 != 0')->all();

        return $this->render('home-site', [
            'model' => $model,
            'tickets' => $tickets,
            'headliner' => $headliner,
            'awards' => $awards,
            'lineup' => $lineup,
            'schedule' => $schedule,
            'daysSchedule' => $days,
            'lineupList' => $lineupList,
            'amazing' => $amazing,
            'amazingTwo' => $amazingTwo,
        ]);
    }

    /**
     * Logs in a user.
     *
     * @return Response|string
     */
    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $this->layout = 'main-auth';

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {

            return $this->redirect('/events/index');
//            return $this->goBack();
        }

        $model->password = '';

        return $this->render('login', [
            'model' => $model,
        ]);
    }

    /**
     * Logs out the current user.
     *
     * @return mixed
     */
    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->redirect('login');
//        return $this->goHome();
    }

    /**
     * Displays contact page.
     *
     * @return mixed
     */
    public function actionContact()
    {
        $model = new ContactForm();
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail(Yii::$app->params['adminEmail'])) {
                Yii::$app->session->setFlash('success', 'Thank you for contacting us. We will respond to you as soon as possible.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error sending your message.');
            }

            return $this->refresh();
        }

        return $this->render('contact', [
            'model' => $model,
        ]);
    }

    public function actionRegistration()
    {

        $model = new RegistrationForm();

        $this->layout = 'main-auth';

        if ($this->request->isPost) {
            if ($model->load($this->request->post())) {

                if ($model->validate() && $model->save()) {

                    Yii::$app->session->setFlash('success', 'Form Submitted successfully, We will contact you soon.');
                    return $this->redirect('registration');
                }

            }
        } else {
            $model->loadDefaultValues();
        }

        return $this->render('registration', [
            'model' => $model,
            'country' => Country::find()->where(['status' => 10])->all(),
        ]);
    }

    /**
     * Signs user up.
     *
     * @return Response|string
     */
    public function actionSignup()
    {

        $this->layout = 'main-auth';

        $model = new SignupForm();
        if ($model->load(Yii::$app->request->post()) && $model->signup()) {
            Yii::$app->session->setFlash('success', 'Thank you for registration. Please check your inbox for verification email.');
            return $this->redirect('signup');
//            return $this->goHome();
        }

        return $this->render('signup', [
            'model' => $model,
        ]);
    }

    /**
     * Requests password reset.
     *
     * @return string|Response
     */
    public function actionRequestPasswordReset()
    {

        $this->layout = 'main-auth';

        $model = new PasswordResetRequestForm();
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail()) {
                Yii::$app->session->setFlash('success', 'Check your email for further instructions.');

                return $this->goHome();
            }

            Yii::$app->session->setFlash('error', 'Sorry, we are unable to reset password for the provided email address.');
        }

        return $this->render('requestPasswordResetToken', [
            'model' => $model,
        ]);
    }

    /**
     * Resets password.
     *
     * @param string $token
     * @return mixed
     * @throws BadRequestHttpException
     */
    public function actionResetPassword($token)
    {

        $this->layout = 'main-auth';

        try {
            $model = new ResetPasswordForm($token);
        } catch (InvalidArgumentException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if ($model->load(Yii::$app->request->post()) && $model->validate() && $model->resetPassword()) {
            Yii::$app->session->setFlash('success', 'New password saved.');

            return $this->goHome();
        }

        return $this->render('resetPassword', [
            'model' => $model,
        ]);
    }

    /**
     * Verify email address
     *
     * @param string $token
     * @throws BadRequestHttpException
     * @return Response
     */
    public function actionVerifyEmail($token)
    {

        $this->layout = 'main-auth';

        try {
            $model = new VerifyEmailForm($token);
        } catch (InvalidArgumentException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }
        if (($user = $model->verifyEmail()) && Yii::$app->user->login($user)) {
            Yii::$app->session->setFlash('success', 'Your email has been confirmed!');

            return $this->redirect('/events/index');
        }

        Yii::$app->session->setFlash('error', 'Sorry, we are unable to verify your account with provided token.');
        return $this->goHome();
    }

    /**
     * Resend verification email
     *
     * @return mixed
     */
    public function actionResendVerificationEmail()
    {

        $this->layout = 'main-auth';

        $model = new ResendVerificationEmailForm();
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail()) {
                Yii::$app->session->setFlash('success', 'Check your email for further instructions.');
                return $this->goHome();
            }
            Yii::$app->session->setFlash('error', 'Sorry, we are unable to resend verification email for the provided email address.');
        }

        return $this->render('resendVerificationEmail', [
            'model' => $model
        ]);
    }
}
