<?php

namespace frontend\controllers;

use common\models\ChangeEmail;
use common\models\ChangePassword;
use common\models\Common;
use frontend\models\User;
use frontend\models\UserSearch;
use Yii;
use yii\web\BadRequestHttpException;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\Response;

/**
 * UserController implements the CRUD actions for User model.
 */
class UserController extends Controller
{
    /**
     * @inheritDoc
     */
    public function behaviors()
    {
        return array_merge(
            parent::behaviors(),
            [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ]
        );
    }

    /**
     * @inheritdoc
     * @throws BadRequestHttpException
     */
    public function beforeAction($action)
    {
        if (Yii::$app->user->isGuest) {
            return $this->redirect(['/login']);
        }

        return parent::beforeAction($action);
    }

    /**
     * Lists all User models.
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new UserSearch();
        $dataProvider = $searchModel->search($this->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single User model.
     * @param string $id ID
     * @return string
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    public function actionProfile()
    {
        return $this->render('view', [
            'model' => $this->findModel(Yii::$app->user->identity->id?? null),
        ]);
    }

    public function actionPassword()
    {

        $model = new ChangePassword();

        if ($this->request->isPost) {
            if ($model->load($this->request->post())) {

                $user = $this->findModel(Yii::$app->user->identity->id?? null);

                if( Common::verifyHashString($model->current_password, $user->password_hash)){

                    $user->password_hash = Common::generateHashString($model->new_password);

                    if( $user->validate() && $user->save()) {

                        Yii::$app->session->setFlash('success', 'Password change successfully');
                    }else{

                        Yii::$app->session->setFlash('error', 'Sorry! Failed to change password, please try again');
                    }

                }else{

                    Yii::$app->session->setFlash('error', 'Sorry! You enter wrong password, please try again');
                }

                return $this->redirect('password');
            }
        }

        return $this->render('password', [
            'model' => $model,
        ]);
    }

    public function actionChangePassword($id){

        $model = $this->findModel($id);

        $password = Common::randomStrings(8);
        $model->password_hash = Common::generateHashString($password);

        if( $model->validate() && $model->save())
        {

            Yii::$app->session->setFlash('success', 'Password change successfully');
            if($this->sendEmail($model, $password))
            {

                Yii::$app->session->setFlash('success', 'Password change and email sent successfully');
            }

            return $this->redirect(['view', 'id' => $id]);
        }

        Yii::$app->session->setFlash('error', 'Sorry! failed to change password, please try again');
        return $this->redirect(['view', 'id' => $id]);
    }
    public function actionEmail()
    {

        $model = new ChangeEmail();

        if ($this->request->isPost) {
            if ($model->load($this->request->post())) {

                $user = $this->findModel(Yii::$app->user->identity->id?? null);

                if( Common::verifyHashString($model->password, $user->password_hash) ){

                    $user->email = $model->email;

                    if($user->validate() && $user->save()){

                        Yii::$app->session->setFlash('success', 'Email change successfully');
                    }else{

                        Yii::$app->session->setFlash('error', 'Sorry! Failed email, please try again');
                    }

                }else{

                    Yii::$app->session->setFlash('error', 'Sorry! You enter wrong password, please try again');
                }

                return $this->redirect('email');
            }

        }

        return $this->render('email', [
            'model' => $model,
        ]);
    }

    /**
     * Creates a new User model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|Response
     */
    public function actionCreate()
    {
        $model = new User();

        if ($this->request->isPost) {
            if ($model->load($this->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }
        } else {
            $model->loadDefaultValues();
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing User model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id ID
     * @return string|Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($this->request->isPost && $model->load($this->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing User model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param string $id ID
     * @return Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the User model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id ID
     * @return User the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = User::findOne(['id' => $id])) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
